using System;
using System.Net;
using System.Net.Sockets;
using System.Text;
using System.Threading.Tasks;

namespace Servidor.TcpGatewayA;
/******************************************************************************
SERVIDOR CIERRA LA CONEXIÓN CHEC LUEGO DEL PRIMER MENSAJE, ESPERANDO QUE
CHEC SE CONECTE NUEVAMENTE.
*/
public class TcpGateway
{
    private const int GatewayPort = 6697;
    private const string PosbcServer = "127.0.0.1";
    private const int PosbcPort = 6698;
    
    public static async Task Main()
    {
        TcpListener gatewayListener = new TcpListener(IPAddress.Any, GatewayPort);
        gatewayListener.Start();
        Console.WriteLine($"Gateway escuchando en el puerto {GatewayPort}...");

        while (true)
        {
            TcpClient checClient = await gatewayListener.AcceptTcpClientAsync();
            Console.WriteLine("CHEC conectado.");
            _ = Task.Run(() => HandleChecClientAsync(checClient));
        }
    }

    private static async Task HandleChecClientAsync(TcpClient checClient)
    {
        using (checClient)
        using (NetworkStream checStream = checClient.GetStream())
        using (TcpClient posbcClient = new TcpClient())
        {
            await posbcClient.ConnectAsync(PosbcServer, PosbcPort);
            Console.WriteLine("Conectado a POSBC.");
            using (NetworkStream posbcStream = posbcClient.GetStream())
            {
                Task receiveFromPosbcTask = ReceiveFromPosbcAndSendToChecAsync(posbcStream, checStream);
                
                while (true)
                {
                    // Leer mensaje desde CHEC
                    byte[] lengthBuffer = new byte[4];
                    int bytesRead = await checStream.ReadAsync(lengthBuffer, 0, 4);
                    if (bytesRead == 0) break; // La conexión se ha cerrado

                    if (BitConverter.IsLittleEndian)
                    {
                        Array.Reverse(lengthBuffer);
                    }

                    int messageLength = BitConverter.ToInt32(lengthBuffer, 0);
                    byte[] messageBuffer = new byte[messageLength];
                    bytesRead = await checStream.ReadAsync(messageBuffer, 0, messageLength);
                    if (bytesRead == 0) break; // La conexión se ha cerrado

                    // Remitir mensaje a POSBC
                    await posbcStream.WriteAsync(lengthBuffer, 0, 4);
                    await posbcStream.WriteAsync(messageBuffer, 0, messageLength);

                    Console.WriteLine($"Mensaje remitido a POSBC: {Encoding.UTF8.GetString(messageBuffer)}");
                }

                await receiveFromPosbcTask;
            }
        }
    }

    private static async Task ReceiveFromPosbcAndSendToChecAsync(NetworkStream posbcStream, NetworkStream checStream)
    {
        byte[] buffer = new byte[4096];

        while (true)
        {
            // Leer longitud del mensaje desde POSBC
            byte[] lengthBuffer = new byte[4];
            int bytesRead = await posbcStream.ReadAsync(lengthBuffer, 0, 4);
            if (bytesRead == 0) break; // La conexión se ha cerrado

            if (BitConverter.IsLittleEndian)
            {
                Array.Reverse(lengthBuffer);
            }

            int messageLength = BitConverter.ToInt32(lengthBuffer, 0);
            byte[] messageBuffer = new byte[messageLength];
            bytesRead = await posbcStream.ReadAsync(messageBuffer, 0, messageLength);
            if (bytesRead == 0) break; // La conexión se ha cerrado

            // Remitir mensaje a CHEC
            await checStream.WriteAsync(lengthBuffer, 0, 4);
            await checStream.WriteAsync(messageBuffer, 0, messageLength);

            Console.WriteLine($"Mensaje remitido a CHEC: {Encoding.UTF8.GetString(messageBuffer)}");
        }
    }
}
